﻿/**
* SphereMusic by Grant Skinner. Sep 15, 2008
* Visit www.gskinner.com/blog for documentation, updates and more free code.
*
*
* Copyright (c) 2008 Grant Skinner, gskinner.com
* 
* Permission is hereby granted, free of charge, to any person
* obtaining a copy of this software and associated documentation
* files (the "Software"), to deal in the Software without
* restriction, including without limitation the rights to use,
* copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the
* Software is furnished to do so, subject to the following
* conditions:
* 
* The above copyright notice and this permission notice shall be
* included in all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
* OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
* HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
* WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
* OTHER DEALINGS IN THE SOFTWARE.
**/

package  {
	
	import flash.display.Shape;
	import flash.display.Sprite;
	import flash.display.MovieClip;
	import fl.motion.easing.*;
	import flash.media.Sound;
	import flash.media.SoundChannel;
	import flash.net.URLRequest;
	import flash.events.Event;
	import flash.geom.Matrix3D;
	import flash.geom.ColorTransform;
	import flash.filters.BlurFilter;
	import flash.events.IOErrorEvent;
	import com.gskinner.motion.GTween;
	
	public class SphereMusic extends MovieClip {
		
		// Constants:
		// Public Properties:
		// Private Properties:
		var circles:Vector.<Circle>;
		var sphere:Sprite;
		var soundChannel:SoundChannel;
		var rotY:Number=0;
		var rotZ:Number=0;
		
		// UI Elements:
		
		// Initialization:
		public function SphereMusic() {
			stage.quality = "low";
			loadMusic();
			buildSphere();
			addEventListener(Event.ENTER_FRAME,tick);
		}

		// Public Methods:
		// Protected Methods:
		// Private Methods:
		private function buildSphere():void {
			circles = new Vector.<Circle>();
			sphere = new Sprite();
			sphere.x = sphere.y = 275;
			sphere.z = 200;
			addChild(sphere);
			
			var count:uint=0;
			for (var i:int=0; i< 8*16; i++) {
				var rX:Number = (i%16);
				var rY:Number = (i/16>>0);
				
				// skip extra circles on and around the poles
				if (rY > 0 && (rX == 0 || rX == 8) || ((rY%2==0 && (rX == 1 || rX == 9 || rX == 7 || rX == 15)))) {
					continue;
				}
				
				// draw circles double size because player 10 will rasterize them:
				var circle:Circle = new Circle(0.5);
				circle.dot1.z = 200;
				circle.dot2.z = 300;
				circle.line1.rotationZ = i/128*360;
				circle.rotationY = rX*(360/16);
				circle.rotationZ = rY*(180/8);
				// slight offset to make it roll:
				circle.z = 100;
				
				circles.push(circle);
				sphere.addChild(circle);
				
				// sphere to disc tweening fun:
				var rnd:Number = rX%8/8;//Math.random();
				var rnd2:Number = count/98*Math.PI*2;
				new GTween(circle,rnd*4+1,{rotationZ:0,rotationY:count/98*360, x:Math.sin(rnd2)*180, z:Math.cos(rnd2)*180+100},{autoReverse:true,delay:4-rnd*4+4,ease:Quadratic.easeInOut});
				
				// also cool, twisty:
				//new GTween(circle,rnd*3+1,{rotationZ:count/98*720-360,rotationY:count/98*360},{autoReverse:true,delay:3-rnd*3+2,ease:Sine.easeIn});
				
				// also also cool, snowflake:
				/*
				var rnd:Number = rX%8/8;//Math.random();
				var rnd2:Number = rnd*Math.PI*2;
				new GTween(circle,rnd*2+1,{rotationZ:0,rotationY:count/98*360, x:Math.sin(rnd2)*180, z:Math.cos(rnd2)*180},{autoReverse:true,delay:2-rnd*2+2,ease:Quadratic.easeInOut});
				*/
				
				count++;
			}
		}
		
		function tick(evt:Event):void {
			var volume:Number = (soundChannel.leftPeak+soundChannel.rightPeak)/2;
			sphere.z = Math.min(3000-volume*3200,sphere.z+40);
			
			rotZ += (soundChannel.leftPeak-0.2)*0.5;
			rotZ *= 0.98;
			rotY += (soundChannel.rightPeak-0.1)*0.4;
			rotY *= 0.97;
			
			sphere.rotationY += rotY;
			sphere.rotationX += (rotZ+rotY)*0.3;
			
			var arr:Array = [];
			for (var i:int=0; i<circles.length; i++) {
				var circle:Circle = circles[i] as Circle;
				
				// depth sort
				var mtx:Matrix3D = circle.dot1.transform.getRelativeMatrix3D(this);
				arr.push({c:circle,z:mtx.position.z});
			}
			
			arr.sortOn("z",Array.NUMERIC|Array.DESCENDING);
			var baseZ:Number = sphere.z;
			for (i=0; i<arr.length; i++) {
				// zsort the circles:
				circle = arr[i].c as Circle;
				var z:Number = arr[i].z;
				sphere.setChildIndex(circle,i);
				
				// swap the dots based on position:
				circle.setChildIndex(circle.dot2,(z > baseZ ? 0 : 1));
				
				// darken circle elements based on distance from the camera:
				var b:Number = Math.min(1, 1+(baseZ-z)/250);
				var o:Number = Math.max(0,(baseZ-z)/200*64);
				circle.transform.colorTransform = new ColorTransform(b,b,b,1,o*0.7,o*0.2,o,0);
				// focus depth blur effect, warning, very slow:
				/*
				b = z/200;
				if (b < 0) { b *= -10; }
				circle.filters = (b > 2) ? [new BlurFilter(b,b,1)] : [];
				*/
			}
		}

		
		private function loadMusic():void {
			var sound:Sound = new Sound(new URLRequest("music.mp3"));
			sound.addEventListener(IOErrorEvent.IO_ERROR,handleError);
			soundChannel = sound.play();
		}
		
		private function handleError(evt:Event):void {}
	}
	
}

import flash.display.Sprite;
import flash.display.Shape;
import flash.filters.GlowFilter;
class Circle extends Sprite {
	
	public var dot1:Shape;
	public var dot2:Shape;
	public var line1:Shape;
	public var line2:Shape;
	
	public function Circle(scale:Number):void {
		dot1 = new Shape();
		dot1.graphics.beginFill(0x9900CC,1);
		dot1.graphics.drawCircle(0,0,10/scale);
		dot1.graphics.endFill();
		dot1.graphics.beginFill(0x000000,1);
		dot1.graphics.drawCircle(0,0,8/scale);
		dot2 = new Shape();
		dot2.graphics.beginFill(0x9900CC,1);
		dot2.graphics.drawCircle(0,0,4/scale);
		line1 = new Shape();
		line1.graphics.beginFill(0x6600FF,0.3);
		line1.graphics.drawRect(-1,0,2,100);
		line1.z = 200;
		line1.rotationX = 90;
		line2 = new Shape();
		line2.graphics.beginFill(0x551FFF,Math.random()*0.5+0.5);
		line2.graphics.drawRect(-1,-25,2,35);
		line2.blendMode = "add";
		line2.z = 0;
		line2.rotationX = line2.rotationY = 90;
		addChild(dot1);
		addChild(dot2);
		addChild(line1);
		addChild(line2);
		dot1.scaleX = dot1.scaleY = dot2.scaleX = dot2.scaleY = scale;
	}
}
